@@ -150,7 +150,10 @@ class Repository(AnyEntity):
             parent = self.branch_head(parentbranch)
         #assert parent
         args = {'description' : msg, 'author': author, 'branch': branch,
-                'repoeid' : self.eid, 'parent': parent and parent.eid}
+                'repoeid' : self.eid, 'parent': parent and parent.eid,
+                # XXX mercurial itself will override (for now) the final phase value
+                'phase': u'secret' if self.type == 'mercurial' else u'public',
+                'hidden': False}
         rset = self._cw.execute(queries.new_revision_rql(parent), args)
         return rset.get_entity(0, 0)
 
@@ -185,8 +188,8 @@ class Revision(AnyEntity):
     """customized class for Revision entities"""
     __regid__ ='Revision'
     fetch_attrs, cw_fetch_order = fetch_config(['revision', 'changeset', 'branch',
-                                             'author', 'description', 'creation_date'],
-                                            order='DESC')
+                                                'author', 'description', 'creation_date'],
+                                               order='DESC')
 
     def dc_title(self):
         if self.changeset:
@@ -221,6 +224,15 @@ class Revision(AnyEntity):
     def children_revisions(self):
         return self.reverse_parent_revision
 
+    def export(self):
+        '''return text/diff version of the revision'''
+        if self.changeset is None:
+            return None
+        return self._cw.call_service('vcs.export-rev',
+                                     repo=self.repository.eid,
+                                     nodeid=self.changeset)
+
+
 
 class VersionedFile(AnyEntity):
     """customized class for VersionedFile entities"""
