@@ -2,9 +2,11 @@
 """hooks for vcsfile content types
 
 :organization: Logilab
-:copyright: 2007-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2007-2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
+from __future__ import with_statement
+
 __docformat__ = "restructuredtext en"
 
 import os
@@ -14,8 +16,9 @@ import shutil
 from logilab.mtconverter import need_guess, guess_mimetype_and_encoding
 
 from cubicweb import QueryError, ValidationError, Binary
-from cubicweb.server import hook, session
-from cubicweb.predicates import is_instance, score_entity, configuration_values
+from cubicweb.server import hook, session, Service
+from cubicweb.predicates import (is_instance, score_entity, configuration_values,
+                                match_user_groups)
 
 from cubes.vcsfile import IMMUTABLE_ATTRIBUTES, bridge
 
@@ -113,56 +116,9 @@ class VCSFileStartHook(hook.Hook):
     events = ('server_startup',)
 
     def __call__(self):
-        repo = self.repo
-        hgrcpath = repo.config['hgrc-path']
-        if hgrcpath is not None:
-            os.environ['HGRCPATH'] = hgrcpath
-        rql = ('Any R, RT, RE, RLC, RSU, RP, RIRC WHERE R is Repository, '
-               'NOT (R local_cache NULL AND R path NULL),'
-               'R type RT, R encoding RE, R local_cache RLC, R source_url RSU, R path RP,'
-               'R import_revision_content RIRC')
-        if not repo.config['cache-external-repositories']:
-            rql += ', R cw_source S, S name "system"'
-        def refresh_local_repo_caches(repo=repo, osp=osp, bridge=bridge, rql=rql):
-            session = repo.internal_session()
-            try:
-                reposrset = session.execute(rql)
-                for vcsrepo in reposrset.entities():
-                    # watch for repository being shut-down, our thread may:
-                    # * not receive the KeyboardInterrupt or shutdown signal
-                    # * loop a while without accessing the session (actually
-                    #   until some repository is actually not up-to-date, while
-                    #   this access to the session would abort the task)
-                    if repo.shutting_down:
-                        break
-                    try:
-                        repohdlr = bridge.repository_handler(vcsrepo)
-                    except bridge.VCSException, ex:
-                        repo.error(str(ex))
-                        continue
-                    except Exception:
-                        repo.exception('error while retrieving handler for %s',
-                                       vcsrepo.eid)
-                        continue
-                    if (repo.config['repository-import'] or vcsrepo.import_revision_content) \
-                           and vcsrepo.local_cache is not None:
-                        try:
-                            repohdlr.pull_or_clone_cache(vcsrepo.source_url)
-                        except bridge.VCSException, ex:
-                            repohdlr.error(str(ex))
-                        except Exception:
-                            repohdlr.exception(
-                                'error while updating local cache of repository %s',
-                                vcsrepo.eid)
-                    if repo.config['repository-import'] and vcsrepo.cw_metainformation()['source']['uri'] == 'system':
-                        bridge.import_vcsrepo_content(
-                            session, repohdlr, vcsrepo,
-                            commitevery=repo.config.get('check-revision-commit-every', 1))
-            finally:
-                session.close()
-
-        repo.looping_task(repo.config.get('check-revision-interval'),
-                          refresh_local_repo_caches)
+        # closure to avoid error on code reloading
+        self.repo.looping_task(self.repo.config.get('check-revision-interval'),
+                               lambda refresh=_refresh_repos, r=self.repo: refresh(r))
 
 
 # internals to be able to create new vcs repo revision using rql queries #######
@@ -312,26 +268,6 @@ class SetRepoCacheBeforeAddHook(hook.Hook):
             set_local_cache(repo)
 
 
-class CloneRepoAfterAddHook(hook.Hook):
-    """clone just after repo creation, to allow operation which needs local
-    access and get quicker checkout if test are running on the same host
-    """
-    __regid__ = 'clone_repo_after_add'
-    __select__ = (hook.Hook.__select__ & is_instance('Repository'))
-    events = ('after_add_entity',)
-    category = 'vcsfile.cache'
-    order = 2
-
-    def __call__(self):
-        vcsrepo = self.entity
-        if not vcsrepo.local_cache:
-            return
-        # XXX rmdir on rollback
-        if (vcsrepo.import_revision_content
-            or self._cw.vreg.config['repository-import']):
-            clone_to_local_cache(vcsrepo)
-
-
 class UpdateRepositoryHook(hook.Hook):
     """add repository eid to vcs bridge cache"""
     __regid__ = 'vcsfile.update_repository_hook'
@@ -501,3 +437,69 @@ class ClassifyVersionedFileHook(hook.Hook):
                 if rset:
                     self._cw.execute('SET X filed_under F WHERE X eid %(x)s, F eid %(f)s',
                                     {'x': self.entity.eid, 'f': rset[0][0]})
+
+
+def _refresh_repos(repo, eids=None, bridge=bridge):
+    '''
+    Refresh vcs repos designated by the `eids` parameter, if refreshable:
+
+      * None for all vcs repos
+      * a list of integers for specific vcs repos
+    '''
+    with repo.internal_session() as session:
+        config = session.vreg.config
+        hgrcpath = repo.config['hgrc-path']
+        if hgrcpath is not None:
+            os.environ['HGRCPATH'] = hgrcpath
+        rql = ('Any R, RT, RE, RLC, RSU, RP, RIRC WHERE R is Repository, '
+               'NOT (R local_cache NULL AND R path NULL),'
+               'R type RT, R encoding RE, R local_cache RLC, R source_url RSU, R path RP,'
+               'R import_revision_content RIRC')
+        if not config['cache-external-repositories']:
+            rql += ', R cw_source S, S name "system"'
+        kwargs = None
+        if eids is not None:
+            if isinstance(eids, int):
+                rql += ', R eid %(x)s'
+                kwargs = {'x': eids}
+            else:
+                rql += ', R eid IN (%s)' % ','.join(str(eid) for eid in eids)
+        reposrset = session.execute(rql, kwargs)
+        for vcsrepo in reposrset.entities():
+            # watch for repository being shut-down, our thread may:
+            # * not receive the KeyboardInterrupt or shutdown signal
+            # * loop a while without accessing the session (actually
+            #   until some repository is actually not up-to-date, while
+            #   this access to the session would abort the task)
+            try:
+                repohdlr = bridge.repository_handler(vcsrepo)
+            except bridge.VCSException, ex:
+                session.error(str(ex))
+                continue
+            except Exception:
+                session.exception('error while retrieving handler for %s',
+                                  vcsrepo.eid)
+                continue
+            if (config['repository-import'] or vcsrepo.import_revision_content) \
+                   and vcsrepo.local_cache is not None:
+                try:
+                    repohdlr.pull_or_clone_cache(vcsrepo.source_url)
+                except bridge.VCSException, ex:
+                    repohdlr.error(str(ex))
+                except Exception:
+                    repohdlr.exception(
+                        'error while updating local cache of repository %s',
+                        vcsrepo.eid)
+            if (config['repository-import']
+                and vcsrepo.cw_metainformation()['source']['uri'] == 'system'):
+                bridge.import_vcsrepo_content(
+                    session, repohdlr, vcsrepo,
+                    commitevery=config.get('check-revision-commit-every', 1))
+
+
+class RefreshRepoService(Service):
+    __regid__ = 'refresh_repository'
+    __select__ = Service.__select__ & match_user_groups('managers')
+
+    def call(self, eids=None):
+        _refresh_repos(self._cw.repo, eids)
