@@ -1,7 +1,7 @@
 """repository abstraction for a mercurial repository
 
 :organization: Logilab
-:copyright: 2007-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2007-2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 
@@ -11,10 +11,11 @@ __docformat__ = "restructuredtext en"
 
 import urllib2
 from os import umask, makedirs
-from os.path import split, exists, join, dirname
+from os.path import split, exists, dirname
 from datetime import datetime
 
-from mercurial import hg, ui, revlog, context, util, commands as hgcmd
+from mercurial import (hg, ui, context, util, encoding,
+                       commands as hgcmd, phases, obsolete)
 from mercurial.node import nullid, short as short_hex
 
 from logilab.common import tempattr
@@ -39,6 +40,32 @@ def spop(aset, item):
     except KeyError:
         pass
 
+# mercurial 2.4 at least does not contain any code to get
+# the precursors ...
+def iter_direct_known_precursors(ctx):
+    obsstore = getattr(ctx._repo, 'obsstore', None)
+    if obsstore is None:
+        return
+    nodemap = ctx._repo.changelog.nodemap
+    markers = obsstore.precursors.get(ctx.node(), ())
+    candidates = set(mark[0] for mark in markers)
+    seen = set(candidates)
+    # we traverse chains of markers which may refer to
+    # nodes not known from the repo (because they were
+    # not pushed/pulled)
+    while candidates:
+        current = candidates.pop()
+        crev = nodemap.get(current)
+        # is the marked revision known from the repo ?
+        if crev is not None:
+            yield ctx._repo[crev]
+            continue
+        # if not, let's continue until we reach a known precursor
+        for mark in obsstore.precursors.get(current, ()):
+            if mark[0] not in seen:
+                candidates.add(mark[0])
+                seen.add(mark[0])
+
 class vcsfile_ui(ui.ui):
     """ redirect output to cubicweb log """
     def __init__(self, loggable):
@@ -96,21 +123,23 @@ class HGRepository(repo.VCSRepository):
             except (util.Abort, urllib2.URLError), ex:
                 raise bridge.VCSException(self.eid, 'path', str(ex))
 
-    def import_content(self, repoentity, commitevery=10):
+    def import_content(self, repoentity, commitevery=1000):
         """import content from the mercurial repository"""
         assert self.eid == repoentity.eid
         if not exists(self.path):
-            self.info('repository %s not accessible in %s',
-                      repoentity.dc_title(), self.path)
-            return
+            self.pull_or_clone_cache(repoentity.source_url)
         try:
             repo = self.hgrepo()
+            # mercurial 2.5
+            if hasattr(repo, 'unfiltered'):
+                repo = repo.unfiltered()
         except Exception, ex:
             self.exception('Impossible to open repository %s (%s)', self, ex)
             return
 
         session = repoentity._cw
-        knownheads = [(str(r.changeset), r.revision) for r in repoentity.heads_rset().entities()]
+        knownheads = [(str(r.changeset), r.revision)
+                      for r in repoentity.heads_rset().entities()]
 
         # detect stripped branches
         needsstrip = False
@@ -123,7 +152,8 @@ class HGRepository(repo.VCSRepository):
         if not needsstrip:
             incoming = repo.revs("not ::%ls", [cset for cset, _rev in knownheads])
             if incoming:
-                needsrenumbering = session.execute('Any R WHERE R is Revision, R from_repository REPO, '
+                needsrenumbering = session.execute('Any R WHERE R is Revision, '
+                                                   'R from_repository REPO, '
                                                    'REPO eid %%(reid)s, R revision IN (%s)'
                                                    % ','.join([str(rev) for rev in incoming]),
                                                    {'reid': repoentity.eid})
@@ -138,7 +168,7 @@ class HGRepository(repo.VCSRepository):
                          repoentity.dc_title(), self.path)
             # clean all 'revision' attribute to please unicity
             # constraint when renumbering
-            session.execute('SET R revision NULL WHERE R from_repository REPO, '
+            session.execute('SET R revision 0-REV WHERE R revision REV, R from_repository REPO, '
                             'REPO eid %(reid)s', {'reid': repoentity.eid})
             revs = session.execute('Any R WHERE R is Revision, R from_repository REPO, '
                                    'REPO eid %(reid)s', {'reid': repoentity.eid})
@@ -149,6 +179,13 @@ class HGRepository(repo.VCSRepository):
                     rev = repo[str(reventity.changeset)].rev()
                     reventity.set_attributes(revision=rev)
 
+            if session.execute('Revision R WHERE R revision < 0, R from_repository REPO, '
+                                   'REPO eid %(reid)s', {'reid': repoentity.eid}):
+                session.rollback()
+                self.error('Abort stripping and renumbering of Repository %s: '
+                           'detected remaining Revision with negative revision number',
+                           repoentity.eid)
+                return
             session.commit()
             session.set_cnxset()
             session.execute(
@@ -159,21 +196,74 @@ class HGRepository(repo.VCSRepository):
             session.commit()
             session.set_cnxset()
 
-        for rev in repo.revs("not ::%ls", [cset for cset, _rev in knownheads]):
-            try:
-                self.import_revision(repoentity, repo, rev)
-            except Exception:
-                self.critical('error while importing revision %s of %s',
-                              rev, self.path, exc_info=True)
-                raise
-            if not rev % commitevery:
-                # give a chance to threads waiting for a cnxset by
-                # freeing it then reacquiring it
-                session.commit()
-                session.set_cnxset()
+        # update phases and visibility of already known revisions
+        self.update_phases(session, repoentity, repo)
+        self.update_hidden(session, repoentity, repo)
+        # XXX we need to update obsolescence markers without new revision
+        #     involved.
+        knownheads = [str(r.changeset) for r in repoentity.heads_rset().entities()]
+        missing = repo.revs("not ::%ls", knownheads)
+        missing.sort()
+        while missing:
+            for rev in missing[:commitevery]:
+                try:
+                    self.import_revision(repoentity, repo, rev)
+                except Exception:
+                    self.critical('error while importing revision %s of %s',
+                                  rev, self.path, exc_info=True)
+                    raise
+            # give a chance to threads waiting for a cnxset by
+            # freeing it then reacquiring it
+            session.commit()
+            session.set_cnxset()
+            del missing[:commitevery] # pop iterated value
+        # if missing was empty we still need to commit phases and visibility changes
+        # give a chance to threads waiting for a cnxset by
+        # freeing it then reacquiring it
         session.commit()
         session.set_cnxset()
 
+    def update_phases(self, session, repoentity, repo):
+        """ update all phases (this potentially affects all revs since
+        phase changes are not transactional) """
+        notpublic = set(str(ctx) for ctx in repo.set('not public()'))
+        if notpublic:
+            session.execute('SET R phase "public" WHERE NOT R changeset IN (%s),'
+                            'NOT R phase "public", R from_repository REPO,'
+                            'REPO eid %%(repo)s' %
+                            ','.join(repr(rev) for rev in notpublic),
+                            {'repo': repoentity.eid})
+            for phase in ('draft', 'secret'):
+                revs = set(str(ctx) for ctx in repo.set('%s()' % phase))
+                if revs:
+                    session.execute('SET R phase "%s" WHERE R changeset IN (%s),'
+                                    'NOT R phase "%s", R from_repository REPO,'
+                                    'REPO eid %%(repo)s' %
+                                    (phase, ','.join(repr(rev) for rev in revs), phase),
+                                    {'repo': repoentity.eid})
+        else:
+            session.execute('SET R phase "public" WHERE R from_repository REPO, '
+                            'NOT R phase "public", REPO eid %(repo)s',
+                            {'repo': repoentity.eid})
+
+
+
+    def update_hidden(self, session, repoentity, repo):
+        """ maintain hidden status for all revs """
+        oldhidden = set(num for num, in session.execute('Any N WHERE R hidden True, '
+                                                        'R from_repository H, '
+                                                        'R revision N, H eid %(hg)s',
+                                                        {'hg': repoentity.eid}).rows)
+        newhidden = set(repo.revs('hidden()'))
+        hide = newhidden - oldhidden
+        show = oldhidden - newhidden
+        for revs, setto in ((hide, 'True'), (show, 'False')):
+            if revs:
+                session.execute('SET R hidden %s WHERE R from_repository H,'
+                                'H eid %%(hg)s, R revision IN (%s)' %
+                                (setto, ','.join(str(revnum) for revnum in revs)),
+                                {'hg': repoentity.eid})
+
     def import_revision(self, repoentity, repo, i):
         self.info('importing revision %s', i)
         session = repoentity._cw
@@ -192,12 +282,25 @@ class HGRepository(repo.VCSRepository):
                    'author': unicode(ctx.user(), self.encoding, 'replace'),
                    'description': unicode(ctx.description(), self.encoding, 'replace'),
                    'changeset': changeset,
+                   'phase': unicode(ctx.phasestr()),
                    'branch': unicode(ctx.branch()),
+                   'hidden': ctx.hidden()
                    }
-        parent_changesets = [short_hex(n) for n in repo.changelog.parents(node)
-                   if n != nullid]
+        parent_changesets = [short_hex(n)
+                             for n in repo.changelog.parents(node)
+                             if n != nullid]
+        precursors_changesets = [str(p) for p in iter_direct_known_precursors(ctx)]
+        if not precursors_changesets:
+            precursors = ()
+        else:
+            precursors = execute(
+                'Any X WHERE X revision XR, X changeset XC, '
+                'X changeset IN (%s), X from_repository R, R eid %%(r)s'
+                % ','.join("'%s'" % cs for cs in precursors_changesets),
+                {'r': self.eid})
+        revdata['precursors'] = [r[0] for r in precursors]
         if not parent_changesets:
-            parents = []
+            parents = ()
         elif len(parent_changesets) == 1:
             parents = execute(
                 'Any X,XC,XR WHERE X revision XR, X changeset XC, '
@@ -267,7 +370,8 @@ class HGRepository(repo.VCSRepository):
                                 'dir': dir, 'name': name}), oldfile
 
     def hgrepo(self):
-        return hg.repository(ui.ui(), self.path)
+        repo = hg.repository(ui.ui(), self.path)
+        return getattr(repo, 'unfiltered', lambda: repo)()
 
     def _file_content(self, path, rev):
         """extract a binary string with the content of the file at `path` for
@@ -342,9 +446,11 @@ class HGTransaction(object):
             self.hgparent = p1 = self.repo.branchtags().get('default')
         if p1:
             if not self.revision.parent_revision:
-                raise ValidationError(self.revision.eid, {'parent_revision': 'missing expected parent'})
+                raise ValidationError(self.revision.eid,
+                                      {'parent_revision': 'missing expected parent'})
             if [short_hex(p1)] != [r.changeset for r in self.revision.parent_revision]:
-                raise ValidationError(None, {None: 'concurrency error, please re-upload the revision'})
+                raise ValidationError(None, {None: ('concurrency error, please '
+                                                    're-upload the revision')})
 
     def commit(self):
         # XXX: make umask configurable
@@ -363,22 +469,11 @@ class HGTransaction(object):
         # ensure mercurial will use configured repo encoding, not locale's
         # encoding
         # XXX modifying module's global is not very nice but I've no other idea
-        try:
-            # mercurial >= 1.3.1
-            from mercurial import encoding
-            encoding.encoding = repohdlr.encoding
-        except ImportError:
-            util._encoding = repohdlr.encoding
+        encoding.encoding = repohdlr.encoding
         ctx = context.memctx(self.repo, (self.hgparent, None), msg,
                              self.changes.keys(), self._filectx, author,
                              extra=self.extra)
-        try:
-            # mercurial < 1.3.1
-            node = self.repo._commitctx(ctx, force=True, force_editor=False,
-                                        empty_ok=True, use_dirstate=False,
-                                        update_dirstate=False)
-        except AttributeError:
-            node = self.repo.commitctx(ctx)
+        node = self.repo.commitctx(ctx)
         # update revision's record to set correct changeset
         #
         # take care, self.repo != self.session.repo (eg mercurial repo instance
@@ -387,6 +482,7 @@ class HGTransaction(object):
         # remove previous cached value to only save the changeset
         with tempattr(revision, 'cw_edited', EditedEntity(revision)) as rev:
             rev.cw_edited['changeset'] = unicode(short_hex(node))
+            rev.cw_edited['phase'] = unicode(phases.phasenames[self.repo[node].phase()])
             source.update_entity(self.session, rev)
         # we need to explicitly commit the connection to the database since this
         # method is actually called from a postcommit event, i.e. once cnxset has
