@@ -1,13 +1,14 @@
 """actions for entity types defined by the vcsfile package
 
 :organization: Logilab
-:copyright: 2007-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2007-2012 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 __docformat__ = "restructuredtext en"
 _ = unicode
 
-from cubicweb.predicates import (one_line_rset, is_instance, rql_condition, score_entity)
+from cubicweb.predicates import (one_line_rset, is_instance, rql_condition,
+                                 score_entity, match_user_groups)
 from cubicweb.web.action import Action
 from cubicweb.web.views.actions import CopyAction
 
@@ -15,6 +16,7 @@ from cubes.vcsfile.entities import rql_revision_content
 
 
 class VCAction(Action):
+    __abstract__ = True
     __select__ = one_line_rset() & is_instance('VersionContent', 'DeletedVersionContent')
     category = 'mainactions'
 
@@ -66,6 +68,20 @@ class VCRevisionModifiedFilesAction(VCAction):
                                   % (reveid, repoeid), vid='list')
 
 
+class RepositoryRefreshAction(Action):
+    __regid__ = 'refresh'
+    __select__ = is_instance('Repository') & match_user_groups('managers')
+
+    title = _('refresh repository')
+    order = 130
+    #category = 'moreactions'
+
+    def url(self):
+        entity = self.cw_rset.get_entity(0, 0)
+        repoeid = entity.eid
+        return self._cw.build_url('refresh_repository', eid=repoeid)
+
+
 class VCRevisionAllFilesAction(VCAction):
     __regid__ = 'revallfiles'
 
@@ -82,6 +98,7 @@ class VCRevisionAllFilesAction(VCAction):
 
 
 class VFAction(Action):
+    __abstract__ = True
     __select__ = one_line_rset() & is_instance('VersionedFile')
     category = 'mainactions'
 
