@@ -289,6 +289,16 @@ class RevisionPrimaryView(primary.PrimaryView):
             self.w(u'</table>')
         else:
             self.w(u'<p>%s</p>' % self._cw._('this revision hasn\'t modified any files'))
+        patch = rev.export()
+        if patch is not None:
+            self.w(u'<h3>Changes</h3>')
+            self.w(u'<div class="content">')
+            transformer = rev._cw_mtc_transform
+            html = transformer(patch, 'text/x-diff', 'text/annotated-html', 'utf8')
+            self.w(html)
+            self.w(u'</div>')
+
+
 
 
 # navigation: breadcrumbs / prevnext adapters ##################################
